<?php

namespace Drupal\cloud_orchestrator\Installer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Additional Cloud Orchestrator related configurations.
 */
class ModuleConfigureForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cloud_orchestrator_module_configure_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $form['#title'] = 'Additional configurations';

    // Mock checkboxes showing required providers.
    $form['required_providers'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled Cloud Service Providers'),
      '#options' => [
        'aws_cloud' => $this->t('AWS'),
        'k8s' => $this->t('K8s'),
      ],
      '#default_value' => [
        'aws_cloud',
        'k8s',
      ],
      '#description' => $this->t('These cloud service providers are enabled by default'),
      '#disabled' => TRUE,
    ];

    $form['cloud_service_providers'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Optional Cloud Service Providers'),
      '#options' => [
        'openstack' => $this->t('OpenStack'),
        'vmware' => $this->t('VMware'),
        'terraform' => $this->t('Terraform'),
      ],
      '#description' => $this->t('Choose optional cloud service provider(s) to enable.'),
    ];

    $form['install_ldap'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Install LDAP modules'),
      '#description' => $this->t('Install LDAP modules to authenticate with Active Directory.  LDAP can also be installed at a later time.'),
    ];

    $form['actions']['submit']['#value'] = $this->t('Save and continue');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\Core\Extension\ModuleInstallerInterface $installer */
    $installer = \Drupal::service('module_installer');

    $modules = array_values(array_filter($form_state->getValue('cloud_service_providers') ?: []));

    // Install any additional providers.
    if ((bool) $form_state->getValue('install_ldap') === TRUE) {
      // Add LDAP modules to install.
      $modules = array_merge($modules, [
        'ldap_user',
        'ldap_servers',
        'ldap_authorization',
        'ldap_query',
        'ldap_authorization',
        'ldap_authentication',
        'authorization',
        'externalauth',
      ]);
    }

    if (!empty($modules)) {
      $installer->install($modules);
    }

    // Set spreadsheet feature to FALSE.
    \Drupal::configFactory()
      ->getEditable('aws_cloud.settings')
      ->set('aws_cloud_instance_type_prices_spreadsheet', FALSE)
      ->save();

    // We install some menu links, so we have to rebuild the router,
    // to ensure the menu links are valid.
    \Drupal::service('router.builder')->rebuildIfNeeded();
    drupal_flush_all_caches();
  }

}
